@@ -53,6 +53,7 @@
 typedef int socklen_t;
 #define SOCK_OPT_TYPE char*
 #include <winsock2.h>
+#include <ws2tcpip.h>
 #endif
 #include <errno.h>
 #include <string.h>
@@ -86,6 +87,10 @@ typedef int socklen_t;
   {    error (#name " is not supported on this platform.");   return octave_value(); };
 
 
+// PKG_ADD: autoload ("AF_UNSPEC", which ("socket"));
+// PKG_DEL: try; autoload ("AF_UNSPEC", which ("socket"), "remove"); catch; end;
+DEFUN_DLD_SOCKET_CONSTANT(AF_UNSPEC );
+
 // PKG_ADD: autoload ("AF_UNIX", which ("socket"));
 // PKG_DEL: try; autoload ("AF_UNIX", which ("socket"), "remove"); catch; end;
 DEFUN_DLD_SOCKET_CONSTANT(AF_UNIX );
@@ -176,10 +181,19 @@ DEFUN_DLD_UNIMPLMENTED_SOCKET_CONSTANT(SO_DONTROUTE );
 // PKG_DEL: try; autoload ("SO_BROADCAST", which ("socket"), "remove"); catch; end;
 DEFUN_DLD_SOCKET_CONSTANT(SO_BROADCAST );
 #else
-#define BROADCAST -1
+#define SO_BROADCAST -1
 DEFUN_DLD_UNIMPLMENTED_SOCKET_CONSTANT(SO_BROADCAST );
 #endif
 
+#ifdef SO_DEBUG
+// PKG_ADD: autoload ("SO_DEBUG", which ("socket"));
+// PKG_DEL: try; autoload ("SO_DEBUG", which ("socket"), "remove"); catch; end;
+DEFUN_DLD_SOCKET_CONSTANT(SO_DEBUG );
+#else
+#define S_DEBUG -1
+DEFUN_DLD_UNIMPLMENTED_SOCKET_CONSTANT(SO_DEBUG );
+#endif
+
 // PKG_ADD: autoload ("SHUT_RD", which ("socket"));
 // PKG_DEL: try; autoload ("SHUT_RD", which ("socket"), "remove"); catch; end;
 DEFUN_DLD_SOCKET_CONSTANT_VALUE(SHUT_RD, 0);
@@ -192,6 +206,62 @@ DEFUN_DLD_SOCKET_CONSTANT_VALUE(SHUT_WR, 1);
 // PKG_DEL: try; autoload ("SHUT_RDWR", which ("socket"), "remove"); catch; end;
 DEFUN_DLD_SOCKET_CONSTANT_VALUE(SHUT_RDWR, 2);
 
+// A few well known IPPROTO_ values
+
+// PKG_ADD: autoload ("IPPROTO_IP", which ("socket"));
+// PKG_DEL: try; autoload ("IPPROTO_IP", which ("socket"), "remove"); catch; end;
+DEFUN_DLD_SOCKET_CONSTANT(IPPROTO_IP);
+
+// PKG_ADD: autoload ("IPPROTO_ICMP", which ("socket"));
+// PKG_DEL: try; autoload ("IPPROTO_ICMP", which ("socket"), "remove"); catch; end;
+DEFUN_DLD_SOCKET_CONSTANT(IPPROTO_ICMP);
+
+// PKG_ADD: autoload ("IPPROTO_TCP", which ("socket"));
+// PKG_DEL: try; autoload ("IPPROTO_TCP", which ("socket"), "remove"); catch; end;
+DEFUN_DLD_SOCKET_CONSTANT(IPPROTO_TCP);
+
+// PKG_ADD: autoload ("IPPROTO_UDP", which ("socket"));
+// PKG_DEL: try; autoload ("IPPROTO_UDP", which ("socket"), "remove"); catch; end;
+DEFUN_DLD_SOCKET_CONSTANT(IPPROTO_UDP);
+
+// AI flags
+
+// PKG_ADD: autoload ("AI_PASSIVE", which ("socket"));
+// PKG_DEL: try; autoload ("AI_PASSIVE", which ("socket"), "remove"); catch; end;
+DEFUN_DLD_SOCKET_CONSTANT(AI_PASSIVE);
+
+// PKG_ADD: autoload ("AI_NUMERICSERV", which ("socket"));
+// PKG_DEL: try; autoload ("AI_NUMERICSERV", which ("socket"), "remove"); catch; end;
+#ifdef AI_NUMERICSERV
+DEFUN_DLD_SOCKET_CONSTANT(AI_NUMERICSERV);
+#else
+DEFUN_DLD_UNIMPLMENTED_SOCKET_CONSTANT(AI_NUMERICSERV );
+#endif
+
+// PKG_ADD: autoload ("AI_V4MAPPED", which ("socket"));
+// PKG_DEL: try; autoload ("AI_V4MAPPED", which ("socket"), "remove"); catch; end;
+#ifdef AI_V4MAPPED
+DEFUN_DLD_SOCKET_CONSTANT(AI_V4MAPPED);
+#else
+DEFUN_DLD_UNIMPLMENTED_SOCKET_CONSTANT(AI_V4MAPPED);
+#endif
+
+// PKG_ADD: autoload ("AI_ADDRCONFIG", which ("socket"));
+// PKG_DEL: try; autoload ("AI_ADDRCONFIG", which ("socket"), "remove"); catch; end;
+#ifdef AI_ADDRCONFIG
+DEFUN_DLD_SOCKET_CONSTANT(AI_ADDRCONFIG);
+#else
+DEFUN_DLD_UNIMPLMENTED_SOCKET_CONSTANT(AI_ADDRCONFIG);
+#endif
+
+// PKG_ADD: autoload ("AI_ALL", which ("socket"));
+// PKG_DEL: try; autoload ("AI_ALL", which ("socket"), "remove"); catch; end;
+#ifdef AI_ALL
+DEFUN_DLD_SOCKET_CONSTANT(AI_ALL);
+#else
+DEFUN_DLD_UNIMPLMENTED_SOCKET_CONSTANT(AI_ALL);
+#endif
+
 //we need to keep track if sockets has been loaded, as it
 //requires initialization on windows platforms.
 #ifdef __WIN32__
@@ -441,12 +511,181 @@ function to disconnect the socket.\n\
   return octave_value (retval);
 }
 
+// PKG_ADD: autoload ("getaddrinfo", which ("socket"));
+// PKG_DEL: try; autoload ("getaddrinfo", which ("socket"), "remove"); ; catch; end;
+// function to get a host information from a host name/service
+DEFUN_DLD(getaddrinfo, args, , "\
+-*- texinfo -*-\n\
+@deftypefn {Loadable Function} {@var{addrinforesults} =} getaddrinfo (@var{hostname}, @var{service}, @var{hints})\n\
+Get addressinfo lookup for a hostname/service.\n\
+\n\
+Returns an array of addressinfo structs with the following fields:\n\
+\n\
+@table @code\n\
+@item family\n\
+Integer family value\n\
+@item socktype\n\
+Integer socktype value\n\
+@item protocol\n\
+Integer protocol value\n\
+@item addr\n\
+a string with the host name to send to\n\
+\n\
+@item port\n\
+the port number to send to (an integer)\n\
+@end table\n\
+\n\
+hints is optional with one or more fields:\n\
+\n\
+@table @code\n\
+@item family\n\
+Integer family value\n\
+@item socktype\n\
+Integer socktype value\n\
+@item protocol\n\
+Integer protocol value\n\
+@item flags\n\
+a integer ai flags\n\
+@end table\n\
+\
+\n\
+See the @command{getaddrinfo} man pages for details.\n\
+\n\
+@end deftypefn")
+{
+  const int nargin = args.length ();
+
+  if (nargin < 2 || nargin > 3)
+    {
+      print_usage ();
+      return octave_value ();
+    }
+
+  if (! args(0).is_string ())
+    {
+      error ("getaddrinfo: HOSTNAME must be a string");
+      return octave_value ();
+    }
+  const std::string addr = args(0).string_value ();
+
+  if (! args(1).is_string ())
+    {
+      error ("getaddrinfo: SERVICE must be a string");
+      return octave_value ();
+    }
+
+  const std::string service = args(1).string_value ();
+ 
+
+  struct addrinfo hints;
+  memset(&hints, 0, sizeof(hints));
+  hints.ai_family = AF_INET;    /* ipv4 */
+  hints.ai_socktype = SOCK_STREAM; /* any type */
+  hints.ai_flags = 0; //AI_PASSIVE;    /* For wildcard IP address */
+  hints.ai_protocol = 0;          /* Any protocol */
+  hints.ai_canonname = NULL;
+  hints.ai_addr = NULL;
+  hints.ai_next = NULL;
+
+  if (nargin > 2 && !args(2).isempty())
+    {
+      const octave_scalar_map struct_hint_info = args(2).scalar_map_value ();
+      if (struct_hint_info.nfields () == 0)
+        {
+          error ("getaddrinfo: HINTS must be a struct");
+          return octave_value ();
+        }
+      // family, socktype, flags, protocol
+      if (struct_hint_info.contains ("family"))
+        {
+          if (struct_hint_info.getfield ("family").is_real_scalar ())
+            hints.ai_family = struct_hint_info.getfield ("family").int_value();
+	  else
+            {
+              error ("getaddrinfo: HINTS.family must be an integer value");
+              return octave_value ();
+            }
+        }
+
+      if (struct_hint_info.contains ("socktype"))
+        {
+          if (struct_hint_info.getfield ("socktype").is_real_scalar ())
+            hints.ai_socktype = struct_hint_info.getfield ("socktype").int_value();
+	  else
+            {
+              error ("getaddrinfo: HINTS.socktype must be an integer value");
+              return octave_value ();
+            }
+        }
+
+      if (struct_hint_info.contains ("flags"))
+        {
+          if (struct_hint_info.getfield ("flags").is_real_scalar ())
+            hints.ai_flags = struct_hint_info.getfield ("flags").int_value();
+	  else
+            {
+              error ("getaddrinfo: HINTS.flags must be an integer value");
+              return octave_value ();
+            }
+        }
+
+      if (struct_hint_info.contains ("protocol"))
+        {
+          if (struct_hint_info.getfield ("protocol").is_real_scalar ())
+            hints.ai_protocol = struct_hint_info.getfield ("protocol").int_value();
+	  else
+            {
+              error ("getaddrinfo: HINTS.protocol must be an integer value");
+              return octave_value ();
+            }
+        }
+    }
+
+  int status;
+  struct addrinfo *result, *rp;
+
+  if ((status = getaddrinfo (
+    addr.empty() ? NULL : addr.c_str(),
+    service.empty() ? NULL : service.c_str(),
+    &hints, &result)) != 0)
+    {
+      error ("getaddrinfo: Failed to get info - error %d", status);
+      return octave_value ();
+    }
+
+  std::vector<octave_value> result_array;
+  for (rp = result; rp != NULL; rp = rp->ai_next)
+    {
+      octave_scalar_map  client_info_map;
+      client_info_map.assign ("family", octave_value (rp->ai_family));
+      client_info_map.assign ("socktype", octave_value (rp->ai_socktype));
+      client_info_map.assign ("protocol", octave_value (rp->ai_protocol));
+
+      // there would normally be addr
+      // TODO: depend on family here
+      sockaddr_in * addr = (sockaddr_in*)rp->ai_addr;
+      client_info_map.assign ("port", octave_value (ntohs(addr->sin_port)));
+      client_info_map.assign ("addr", octave_value (inet_ntoa(addr->sin_addr)));
+      result_array.push_back(octave_value(client_info_map));
+    }
+ 
+  freeaddrinfo(result); // free the linked-list
+
+  Cell cell(result_array.size(), 1);
+  for(octave_idx_type i=0; i<(octave_idx_type)result_array.size(); i++)
+    cell(i, 0) = result_array[i];
+    
+  octave_value_list ret;
+  ret(0) = cell;
+  return ret;
+}
+ 
 // PKG_ADD: autoload ("gethostbyname", which ("socket"));
 // PKG_DEL: try; autoload ("gethostbyname", which ("socket"), "remove"); ; catch; end;
 // function to get a host number from a host name
 DEFUN_DLD(gethostbyname, args, , "\
 -*- texinfo -*-\n\
-@deftypefn {Loadable Function} {@var{ipaddres} =} gethostbyname (@var{hostname})\n\
+@deftypefn {Loadable Function} {@var{ipaddress} =} gethostbyname (@var{hostname})\n\
 Return IP address for host name.\n\
 \n\
 For example:\n\
@@ -1114,7 +1353,8 @@ See the @command{getsockopt} man pages for further details.\n\
   DWORD ivalue = 0;
 #endif
 
-  if (optname == SO_KEEPALIVE || optname == SO_REUSEADDR || optname == SO_TYPE || optname == SO_DONTROUTE || optname == SO_BROADCAST)
+  if (optname == SO_KEEPALIVE || optname == SO_REUSEADDR || optname == SO_TYPE || 
+      optname == SO_DONTROUTE || optname == SO_BROADCAST || optname == SO_DEBUG)
     {
       optlen = sizeof(ivalue);
       optvalue = (SOCK_OPT_TYPE)&ivalue;
@@ -1131,7 +1371,8 @@ See the @command{getsockopt} man pages for further details.\n\
 
   if(result == 0)
     {
-      if (optname == SO_KEEPALIVE || optname == SO_REUSEADDR || optname == SO_TYPE || optname == SO_DONTROUTE || optname == SO_BROADCAST)
+      if (optname == SO_KEEPALIVE || optname == SO_REUSEADDR || optname == SO_TYPE ||
+          optname == SO_DONTROUTE || optname == SO_BROADCAST || optname == SO_DEBUG)
         value = octave_value(ivalue);
     }
 
@@ -1200,7 +1441,8 @@ See the @command{setsockopt} man pages for further details.\n\
   DWORD ivalue = 0;
 #endif
 
-  if (optname == SO_KEEPALIVE || optname == SO_REUSEADDR || optname == SO_DONTROUTE || optname == SO_BROADCAST)
+  if (optname == SO_KEEPALIVE || optname == SO_REUSEADDR || optname == SO_DONTROUTE ||
+      optname == SO_BROADCAST || optname == SO_DEBUG)
     {
       if (! args(3).is_real_scalar ())
         {
